//==================================================================
//
//	File name	:	button.js
//
//	Summary		:	Button class
//
//	Ceated		:	2018/11/19
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (button) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	button.STYLE_DEFAULT = 0;
	button.STYLE_FLAT = 1;
	button.STYLE_TEXT = 2;
	button.STYLE_IMAGE = 4;
	button.STYLE_ICON = 8;
	
	
	//==========================================================
	//
	//	Class		:	Button
	//
	//	Summary		:	Class of button
	//
	//==========================================================
	button.Button = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// * ID[string]
					oRect,			// * Size and position[FixedRect]
					iStyle,			// * Item style[int]
					bEnabled,		// * Enabled or not[boolean]
					sContents,		// Label text or image URL or icon name[string]
					sTooltip,		// Tool tip text[string]
					iTabIndex,		// Tab index[int]
					fpClick)		// Function on click[function]
		{
			// Base class
			super(false, sID, false, oRect, bEnabled, iTabIndex);
			
			// Check parameters
			if(typeof iStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
											"button.js", "jaswin.button.Button.constructor");
			if(sContents)
			{
				if(typeof sContents != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, sID),
												"button.js", "jaswin.button.Button.constructor");
			}
			
			if(sTooltip)
			{
				if(typeof sTooltip != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
												"button.js", "jaswin.button.Button.constructor");
			}
			
			if(fpClick)
			{
				if(typeof fpClick != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, sID),
												"button.js", "jaswin.button.Button.constructor");
			}
			
			// Set properties
			this.style = iStyle;			// Button style[int]
			this.onClick = fpClick;			// Function on click[function(Argument:Button class)]
			this.content = null;			// Content element for text or image[DOM Element]
			
			// Create element
			this.element = document.createElement("div");
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat";
			else
				this.element.className = "button";
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			
			this.element.style.width = this.rect.width - 2 + "px";		// 2:border
			this.element.style.height = this.rect.height - 2 + "px";	// 2:border
			this.focusElement = this.element;
			
			if(!this.enabled)
			this.element.style.cursor = "default";
			
			// Create contents
			if(this.style & button.STYLE_TEXT)
			{
				// Text label
				this.content = document.createElement("span");
				if(this.enabled)
					this.content.className = "button-text";
				else
					this.content.className = "button-text-disabled";
				
				this.content.style.lineHeight = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
				this.content.innerHTML = sContents;
				this.element.appendChild(this.content);
			}
			else if(this.style & button.STYLE_IMAGE)
			{
				// Image
				this.content = document.createElement("img");
				if(this.enabled)
					this.content.className = "button-image";
				else
					this.content.className = "button-image-disabled";
				this.content.src = sContents;
				this.element.appendChild(this.content);
			}
			else if(this.style & button.STYLE_ICON)
			{
				// Icon
				this.content = document.createElement("span");
				if(this.enabled)
					this.content.className = "button-icon";
				else
					this.content.className = "button-icon-disabled";
				this.content.style.width = this.rect.width - 2 - 2 + "px";		// 2:border 2:margin
				this.content.style.height = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
				
				let oIconSpan = document.createElement("span");
				oIconSpan.className = sContents;
				
				this.content.appendChild(oIconSpan);
				this.element.appendChild(this.content);
			}
			
			// Tooltip
			if((sTooltip) && (sTooltip.length > 0))
				this.element.title = sTooltip;
			
			// Tab index
			if((iTabIndex) && (iTabIndex > 0))
				this.focusElement.tabIndex = this.tabIndex;
			
			// Event binding
			if(this.enabled)
			{
				this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
					this.setHover();
				}, false);
				this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
					this.setOut();
				}, false);
				this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
					this.setDown();
				}, false);
				this.element.addEventListener("mouseup", this.mouseUpEvent = (e) => {
					this.setUp();
				}, false);
				this.element.addEventListener("click", this.clickEvent = (e) => {
					this.click();
				}, false);
				this.element.addEventListener("keyup", this.keyUpEvent = (e) => {
					if((e.keyCode != 13) && (e.keyCode != 32))
						return true;
					this.click();
				}, false);
				this.focusElement.addEventListener("focus", this.focusEvent = (e) => {
					if((!this.enabled) || (!this.parentEnabled))
					{
						e.preventDefault();
						return;
					}
					this.setHover();
				}, false);
				this.focusElement.addEventListener("blur", this.blurEvent = (e) => {
					if((!this.enabled) || (!this.parentEnabled))
					{
						e.preventDefault();
						return;
					}
					this.setOut();
				}, false);
			}
		}
		
		//==================================================
		//	Enable this button
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if(this.enabled)
			{
				if(this.content)
				{
					if(this.style & button.STYLE_TEXT)
						this.content.className = "button-text";
					else if(this.style & button.STYLE_IMAGE)
						this.content.className = "button-image";
					else if(this.style & button.STYLE_ICON)
						this.content.className = "button-icon";
				}
				
				this.element.style.cursor = "pointer";
				
				if(!this.mouseMoveEvent)
				{
					this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
						this.setHover();
					}, false);
				}
				if(!this.mouseOutEvent)
				{
					this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
						this.setOut();
					}, false);
				}
				if(!this.mouseDownEvent)
				{
					this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
						this.setDown();
					}, false);
				}
				if(!this.mouseUpEvent)
				{
					this.element.addEventListener("mouseup", this.mouseUpEvent = (e) => {
						this.setUp();
					}, false);
				}
				if(!this.clickEvent)
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						this.click();
					}, false);
				}
				if(!this.keyUpEvent)
				{
					this.element.addEventListener("keyup", this.keyUpEvent = (e) => {
						if((e.keyCode != 13) && (e.keyCode != 32))
							return true;
						this.click();
					}, false);
				}
				if(!this.focusEvent)
				{
					this.focusElement.addEventListener("focus", this.focusEvent = (e) => {
						if((!this.enabled) || (!this.parentEnabled))
						{
							e.preventDefault();
							return;
						}
						this.setHover();
					}, false);
				}
				if(!this.blurEvent)
				{
					this.focusElement.addEventListener("blur", this.blurEvent = (e) => {
						if((!this.enabled) || (!this.parentEnabled))
						{
							e.preventDefault();
							return;
						}
						this.setOut();
					}, false);
				}
			}
		}
		
		//==================================================
		//	Disable this button
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat";
			else
				this.element.className = "button";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text-disabled";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image-disabled";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon-disabled";
			}
			
			this.element.style.cursor = "default";
			
			if(this.mouseMoveEvent)
			{
				this.element.removeEventListener("mousemove", this.mouseMoveEvent, false);
				this.mouseMoveEvent = null;
			}
			if(this.mouseOutEvent)
			{
				this.element.removeEventListener("mouseout", this.mouseOutEvent, false);
				this.mouseOutEvent = null;
			}
			if(this.mouseDownEvent)
			{
				this.element.removeEventListener("mousedown", this.mouseDownEvent, false);
				this.mouseDownEvent = null;
			}
			if(this.mouseUpEvent)
			{
				this.element.removeEventListener("mouseup", this.mouseUpEvent, false);
				this.mouseUpEvent = null;
			}
			if(this.clickEvent)
			{
				this.element.removeEventListener("click", this.clickEvent, false);
				this.clickEvent = null;
			}
			if(this.keyUpEvent)
			{
				this.element.removeEventListener("keyup", this.keyUpEvent, false);
				this.keyUpEvent = null;
			}
			if(this.focusEvent)
			{
				this.focusElement.removeEventListener("focus", this.focusEvent, false);
				this.focusEvent = null;
			}
			if(this.blurEvent)
			{
				this.focusElement.removeEventListener("blur", this.blurEvent, false);
				this.blurEvent = null;
			}
		}
		
		//==================================================
		//	Set this button hover
		//==================================================
		setHover()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-focus";
			else
				this.element.className = "button-focus";
		}
		
		//==================================================
		//	Set this button mouse out
		//==================================================
		setOut()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat";
			else
				this.element.className = "button";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon";
			}
		}
		
		//==================================================
		//	Set this button mouse down
		//==================================================
		setDown()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-click";
			else
				this.element.className = "button-click";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text-click";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image-click";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon-click";
			}
		}
		
		//==================================================
		//	Set this button mouse up
		//==================================================
		setUp()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-focus";
			else
				this.element.className = "button-focus";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon";
			}
		}
		
		//==================================================
		//	Click this button
		//==================================================
		click()
		{
			if((this.enabled) && (this.parentEnabled) && (this.onClick))
				this.onClick(this);
		}
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel(sText)
		{
			if(!this.content)
				return;
			
			if(!this.style & button.STYLE_TEXT)
				return;
			
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"button.js", "jaswin.button.Button.setLabel");
			
			this.content.innerHTML = sText;
		}
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"button.js", "jaswin.button.Button.setTooltip");
			this.element.title = sTooltip;
		}
		
		//==================================================
		//	Set click event
		//==================================================
		setClickEvent(fpClick)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, this.id),
											"button.js", "jaswin.button.Button.setClickEvent");
			this.onClick = fpClick;
		}
	}
	
}(jaswin.button = jaswin.button || {}));